//===================================RISCKER====================================
//===================================Divider====================================
//------------------------------------------------------------------------------
//File Name:	divider.v
//Purpose:		Parallel division main module
//Parameters:	numbit (data width)
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module divider # (parameter numbit = 8)( dividend, divisor, result);

localparam w = numbit * 2;
	 
	 input [numbit-1:0] dividend, divisor;
	 output [w-1:0] result;

wire [w-1:0] r [numbit:0];

wire [w-1:0] divisor_shifted16 = {divisor,{numbit{1'b0}}};
wire [w-1:0] dividend_32 = {{numbit{1'b0}},dividend};

wire [w-1:0] c;
genvar j;

generate
for (j=0; j<numbit+1; j=j+1)
		begin:div
			
			if (j==0)
			
			adderB #(w) first_line (.Cin(1'b1),.Sum1(dividend_32),.Sum2(~divisor_shifted16),.Result(r[j]),.Cout(c[numbit+1-j]));
			
			else
			
			adderB #(w) second_line (.Cin(1'b1),.Sum1(r[j-1]),.Sum2(~{{j{1'b0}},divisor_shifted16[w-1:j]}), .Result(r[j]),.Cout(c[numbit+1-j]));
			
		end
		


endgenerate

wire [numbit-1:0] quotient = ~c[numbit:1];
wire [numbit-1:0] remainder = r[numbit][numbit-1:0];

assign result = {remainder, quotient};
endmodule
