//===================================RISCKER====================================
//====================================Timer=====================================
//------------------------------------------------------------------------------
//File Name:	timer.v
//Purpose:		Configurable Timer module and overflow interruption
//Parameters:	width
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module timer #(parameter width = 16)
	(
	input clk, 
	input [1:0] prescaler, 
	input[width-1:0] period, 
	input enable,
	input reset_tmr,
	output reg int_timer,
	output reg [width-1:0] counter=0 //Timer Value
	);

reg [1:0] counter_clk=0;		//Prescaler counter value
reg clk_timer_f=0;				//Clock divided by prescaler

//Clock divider counter
always @ (negedge clk)
		if(counter_clk == prescaler)
			counter_clk <= 0;
		else
			counter_clk <= counter_clk + 1;
	
//Clock generation
always@(posedge clk)
	if(counter_clk == prescaler)
		clk_timer_f <= ~clk_timer_f;
	else
		clk_timer_f<=clk_timer_f;
	
//Timer Counter
always @ (posedge clk_timer_f)
	begin
		if(int_timer|reset_tmr)
			counter <= 0;
		else
			if(enable)
				counter <= counter + 1;
			else
				counter <= counter;
	end

//Timer Interruption
always@(posedge clk)
int_timer <= (counter == period);

endmodule
